// 
// lat - GtkLdapAttributeEditor.cs
// Author: Loren Bandiera
// Copyright 2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.IO;
using Gtk;
using GLib;
using Novell.Directory.Ldap;
using Novell.Directory.Ldap.Utilclass;

namespace lat
{	
	public class GtkLdapAttributeEditor : Gtk.Bin
	{
		protected ListStore store;
		protected Connection conn;
		protected string currentDN;
		protected bool displayAll;		
		protected List<string> allAttrs;
		protected NameValueCollection currentAttributes;
		
		protected Gtk.TreeView treeview2;
		protected Gtk.Button applyButton;

		
		public GtkLdapAttributeEditor()
		{
			Stetic.Gui.Build(this, typeof(lat.GtkLdapAttributeEditor));
			
			store = new ListStore (typeof (string), typeof(string));
			store.SetSortColumnId (0, SortType.Ascending);			
			treeview2.Model = store;
			
			TreeViewColumn col;
			col = treeview2.AppendColumn ("Name", new CellRendererText (), "text", 0);
			col.SortColumnId = 0;

			CellRendererText cell = new CellRendererText ();
			cell.Editable = true;
			cell.Edited += new EditedHandler (OnAttributeEdit);
			
			treeview2.AppendColumn ("Value", cell, "text", 1);
			treeview2.ButtonPressEvent += new ButtonPressEventHandler (OnRightClick);
			
			applyButton.Sensitive = false;
		}

		public void Show (Connection connection, LdapEntry entry, bool showAll)
		{
			displayAll = showAll;
			conn = connection;
			currentDN = entry.DN;			
			currentAttributes = new NameValueCollection ();

			store = new ListStore (typeof (string), typeof(string));
			store.SetSortColumnId (0, SortType.Ascending);
			treeview2.Model = store;
			
			allAttrs = new List<string> ();
			LdapAttribute a = entry.getAttribute ("objectClass");
			
			for (int i = 0; i < a.StringValueArray.Length; i++) {
			
				string o = (string) a.StringValueArray[i];	
				store.AppendValues ("objectClass", o);
				currentAttributes.Add ("objectClass", o);
				
				string[] attrs = conn.Data.GetAllAttributes (o);
				if (attrs != null) {
					foreach (string at in attrs)
						if (!allAttrs.Contains (at))
							allAttrs.Add (at);
				} else {
					Log.Debug("Could not retrieve any attribute for objectClass " + o);
				}
			}
			
			LdapAttributeSet attributeSet = entry.getAttributeSet ();
			CheckForFedoraAciAttributes (attributeSet);			
			DisplayAttributeSet (attributeSet);
			
			if (!showAll)
				return;

			foreach (string n in allAttrs)
				store.AppendValues (n, "");		
		}

		public string CurrentAttributeName
		{
			get { 
				TreeModel model;
				TreeIter iter;
				string name = null;

				if (treeview2.Selection.GetSelected (out model, out iter)) {
					name = (string) store.GetValue (iter, 0);
					return name;
				}

				return name;
			}
		}

		#region internal_methods
		
		void AddBinaryAttribute (string fileName)
		{
			byte[] fileBytes = ReadFileBytes (fileName);
			if (fileBytes.Length == 0)
				return;
				
			string attributeName = this.CurrentAttributeName;
			string attributeValue = Base64.encode (SupportClass.ToSByteArray (fileBytes));
				
			LdapEntry le = conn.Data.GetEntry (currentDN);
			LdapAttribute la = le.getAttribute (attributeName);
			
			bool existing = false;
			if (la != null)
				existing = true; 

			LdapAttribute newla = new LdapAttribute (attributeName);		
			newla.addBase64Value (attributeValue);
				
			LdapModification lm;
			
			if (existing)
				lm = new LdapModification (LdapModification.REPLACE, newla);
			else
				lm = new LdapModification (LdapModification.ADD, newla);

			List<LdapModification> modList = new List<LdapModification> ();
			modList.Add (lm);
				
			Util.ModifyEntry (conn, currentDN, modList.ToArray());
				
			this.Show (conn, conn.Data.GetEntry (currentDN), displayAll);		
		}

		void CheckForFedoraAciAttributes (LdapAttributeSet attributeSet)
		{		
			// Fedora Directory Server supports an Access Control Item (ACI)
			// but it is not listed as "allowed attribute" for any objectClass 
			// found in Fedora's LDAP schema.
			if (displayAll && conn.Settings.ServerType == LdapServerType.FedoraDirectory) {
				LdapEntry[] acientries = conn.Data.Search(currentDN, 
														LdapConnection.SCOPE_BASE,
						    							"objectclass=*", 
						    							new string[] {"aci"} );
						    							
				if (acientries.Length > 0) {
					LdapEntry acientry = acientries[0];
					LdapAttribute aciattr = acientry.getAttribute("aci");
					if (aciattr != null)
						attributeSet.Add(aciattr);
				}
			}		
		}

		void DisplayAttributeSet (LdapAttributeSet attributeSet)
		{
			foreach (LdapAttribute attr in attributeSet) {

				if (allAttrs.Contains (attr.Name))
					allAttrs.Remove (attr.Name);

				if (attr.Name.ToLower() == "objectclass")
					continue;

				try {
				
					foreach (string s in attr.StringValueArray) {
						store.AppendValues (attr.Name, s);
						currentAttributes.Add (attr.Name, s);
					}
					
				} catch (ArgumentOutOfRangeException e) {					
					// FIXME: this only happens with gmcs
					store.AppendValues (attr.Name, "");
					Log.Debug ("Show attribute arugment out of range: {0}", attr.Name);
					Log.Debug (e.Message);
				}
			}		
		}
		
		void DeleteAttribute ()
		{
			TreeModel model;
			TreeIter iter;

			if (!treeview2.Selection.GetSelected (out model, out iter))
				return;
				
			store.Remove (ref iter);
			applyButton.Sensitive = true;
		}
		
		void DoPopUp()
		{
			Menu popup = new Menu();

			ImageMenuItem addBinaryValueItem = new ImageMenuItem ("Add binary value...");
			addBinaryValueItem.Image = new Gtk.Image (Stock.Open, IconSize.Menu);
			addBinaryValueItem.Activated += new EventHandler (OnAddBinaryValueActivate);
			addBinaryValueItem.Show ();
			popup.Append (addBinaryValueItem);

			ImageMenuItem newObjectClassItem = new ImageMenuItem ("Add object class(es)...");
			newObjectClassItem.Image = new Gtk.Image (Stock.Add, IconSize.Menu);
			newObjectClassItem.Activated += new EventHandler (OnAddObjectClassActivate);
			newObjectClassItem.Show ();
			popup.Append (newObjectClassItem);

			ImageMenuItem deleteItem = new ImageMenuItem ("Delete attribute");
			deleteItem.Image = new Gtk.Image (Stock.Delete, IconSize.Menu);
			deleteItem.Activated += new EventHandler (OnDeleteActivate);
			deleteItem.Show ();
			popup.Append (deleteItem);

			ImageMenuItem newItem = new ImageMenuItem ("Insert attribute");
			newItem.Image = new Gtk.Image (Stock.New, IconSize.Menu);
			newItem.Activated += new EventHandler (OnInsertActivate);			
			newItem.Show ();
			popup.Append (newItem);

			ImageMenuItem saveBinaryValueItem = new ImageMenuItem ("Save binary value...");
			saveBinaryValueItem.Image = new Gtk.Image (Stock.Save, IconSize.Menu);
			saveBinaryValueItem.Activated += new EventHandler (OnSaveBinaryValueActivate);
			saveBinaryValueItem.Show ();
			popup.Append (saveBinaryValueItem);

			popup.Popup(null, null, null, 3, Gtk.Global.CurrentEventTime);
		}
		
		void InsertAttribute ()
		{
			string attrName;
			TreeModel model;
			TreeIter iter;

			if (!treeview2.Selection.GetSelected (out model, out iter))
				return;
				
			attrName = (string) store.GetValue (iter, 0);
						
			if (attrName == null)
				return;
								
			SchemaParser sp = conn.Data.GetAttributeTypeSchema (attrName);
			
			if (!sp.Single) {
				TreeIter newRow = store.InsertAfter (iter);
				store.SetValue (newRow, 0, attrName);
				applyButton.Sensitive = true;
			} else {
			
				HIGMessageDialog dialog = new HIGMessageDialog (
					null,
					0,
					Gtk.MessageType.Info,
					Gtk.ButtonsType.Ok,
					"Unable to insert value",
					"Multiple values not supported by this attribute");

				dialog.Run ();
				dialog.Destroy ();
			}						
		}
		
		byte[] ReadFileBytes (string fileName)
		{
			List<byte> fileBytes = new List<byte> ();
		
			try {
							
				FileStream fs = File.OpenRead (fileName);
				
				byte[] buf = new byte[4096];
				int ret = 0;
				
				do {
				
					ret = fs.Read (buf, 0, buf.Length);
					for (int i = 0; i < ret; i++)
						fileBytes.Add (buf[i]);
				
				} while (ret != 0);
				
				fs.Close ();				
			
			} catch (Exception e) {
				Log.Debug (e);
				Util.ShowErrorDialog ("Add binary value error", e.Message);				
			}
			
			return fileBytes.ToArray();
		}
		
		void RunViewerPlugin (AttributeViewPlugin avp, string attributeName)
		{
			LdapEntry le = conn.Data.GetEntry (currentDN);
			LdapAttribute la = le.getAttribute (attributeName);
			
			bool existing = false;
			if (la != null)
				existing = true; 

			LdapAttribute newla = new LdapAttribute (attributeName);
									
			switch (avp.DataType) {
			
			case ViewerDataType.Binary:
				if (existing)
					avp.OnActivate (attributeName, SupportClass.ToByteArray (la.ByteValue));
				else
					avp.OnActivate (attributeName, new byte[0]);
				break;

			case ViewerDataType.String:
				if (existing)
					avp.OnActivate (attributeName, la.StringValue);
				else
					avp.OnActivate (attributeName, "");
				break;				
			}
		
			if (avp.ByteValue != null)
				newla.addBase64Value (System.Convert.ToBase64String (avp.ByteValue, 0, avp.ByteValue.Length));
			else if (avp.StringValue != null)
				newla.addValue (avp.StringValue);
			else
				return;
			
			LdapModification lm;
			if (existing)
				lm = new LdapModification (LdapModification.REPLACE, newla);
			else
				lm = new LdapModification (LdapModification.ADD, newla);

			List<LdapModification> modList = new List<LdapModification> ();
			modList.Add (lm);			
			Util.ModifyEntry (conn, currentDN, modList.ToArray());

			this.Show (conn, conn.Data.GetEntry (currentDN), displayAll);
		}
		
		void WriteBytesToFile (string fileName, byte[] fileBytes)
		{
			try {
							
				FileStream fs = File.OpenWrite (fileName);
				fs.Write (fileBytes, 0, fileBytes.Length);			
				fs.Close ();				
			
			} catch (Exception e) {
				Log.Debug (e);
				Util.ShowErrorDialog ("Save binary attribute error", e.Message);				
			}
		}
		
		#endregion		

		void OnAddBinaryValueActivate (object o, EventArgs args)
		{
			FileChooserDialog fcd = new FileChooserDialog (
				Mono.Unix.Catalog.GetString ("Select file to add as binary attribute"),
				Gtk.Stock.Open, 
				null, 
				FileChooserAction.Open);

			fcd.AddButton (Gtk.Stock.Cancel, ResponseType.Cancel);
			fcd.AddButton (Gtk.Stock.Open, ResponseType.Ok);

			fcd.SelectMultiple = false;

			ResponseType response = (ResponseType) fcd.Run();
			if (response == ResponseType.Ok)
				AddBinaryAttribute (fcd.Filename);

			fcd.Destroy();
		}

		void OnAddObjectClassActivate (object o, EventArgs args)
		{
			AddObjectClassDialog dlg = new AddObjectClassDialog (conn);
			dlg.Run ();
			
			foreach (string s in dlg.ObjectClasses) {
				string[] req = conn.Data.GetRequiredAttrs (s);
				store.AppendValues ("objectClass", s);
				
				foreach (string r in req) {
					if (allAttrs.Contains (r))
						allAttrs.Remove (r);						

					string m = currentAttributes[r];
					if (m == null) {
						store.AppendValues (r, "");
						currentAttributes.Add (r, "");
					}
				}
			}
			
			dlg.Destroy ();
		}

		protected virtual void OnApplyClicked(object sender, System.EventArgs e)
		{
			List<LdapModification> modList = new List<LdapModification> ();
			NameValueCollection newAttributes = new NameValueCollection ();
			
			foreach (object[] row in this.store) {
			
				string newValue = row[1].ToString();
				
				if (newValue == "" || newValue == null)
					continue;
					
				newAttributes.Add (row[0].ToString(), newValue);
			}

			foreach (string key in newAttributes.AllKeys) {
			
				string[] newValues = newAttributes.GetValues(key);
				string[] oldValues = currentAttributes.GetValues (key);
				LdapAttribute la = new LdapAttribute (key, newValues);
				
				if (oldValues == null) {					
					LdapModification lm = new LdapModification (LdapModification.ADD, la);
					modList.Add (lm);
				} else {
										
					foreach (string nv in newValues) {
					
						bool foundMatch = false;
						foreach (string ov in oldValues)
							if (ov == nv)
								foundMatch = true;
								
						if (!foundMatch) {
							LdapModification lm = new LdapModification (LdapModification.REPLACE, la);
							modList.Add (lm);
						}
					}
				}				
			}

			foreach (string key in currentAttributes.AllKeys) {
				string[] newValues = newAttributes.GetValues (key);
								
				if (newValues == null) {
					string[] oldValues = currentAttributes.GetValues (key);
					LdapAttribute la = new LdapAttribute (key, oldValues);
					LdapModification lm = new LdapModification (LdapModification.DELETE, la);
					modList.Add (lm);
				} else {
					LdapAttribute la = new LdapAttribute (key, newValues);
					LdapModification lm = new LdapModification (LdapModification.REPLACE, la);
					modList.Add (lm);
				}
			}

			Util.ModifyEntry (conn, currentDN, modList.ToArray());		
		}

		void OnAttributeEdit (object o, EditedArgs args)
		{
			TreeIter iter;

			if (!store.GetIterFromString (out iter, args.Path))
				return;

			string oldText = (string) store.GetValue (iter, 1);
			if (oldText == args.NewText)
				return;
				
			store.SetValue (iter, 1, args.NewText);
			applyButton.Sensitive = true;
		}

		void OnDeleteActivate (object o, EventArgs args)
		{
			DeleteAttribute ();
		}

		void OnInsertActivate (object o, EventArgs args)
		{
			InsertAttribute ();
		}

		protected virtual void OnKeyPress(object o, Gtk.KeyPressEventArgs args)
		{
			switch (args.Event.Key) {
			
			case Gdk.Key.Insert:
			case Gdk.Key.KP_Insert:
				InsertAttribute ();
				break;
				
			case Gdk.Key.Delete:
			case Gdk.Key.KP_Delete:
				DeleteAttribute ();
				break;
				
			default:
				break;
			}			
		}

		[ConnectBefore]
		protected void OnRightClick (object o, ButtonPressEventArgs args)
		{
			if (args.Event.Button == 3)
				DoPopUp ();
		}

		protected virtual void OnRowActivated(object o, Gtk.RowActivatedArgs args)
		{
			TreePath path = args.Path;
			TreeIter iter;
			
			if (store.GetIter (out iter, path)) {
			
				string name = (string) store.GetValue (iter, 0);					
				foreach (AttributeViewPlugin avp in Global.Plugins.AttributeViewPlugins) {
					foreach (string an in avp.AttributeNames)
						if (an.ToLower() == name.ToLower())
							if (conn.AttributeViewers.Contains (avp.GetType().ToString()))
								RunViewerPlugin (avp, name);
				}
			}		
		}

		void OnSaveBinaryValueActivate (object o, EventArgs args)
		{
			FileChooserDialog fcd = new FileChooserDialog (
				Mono.Unix.Catalog.GetString ("Save binary as"),
				Gtk.Stock.Save, 
				null, 
				FileChooserAction.Save);

			fcd.AddButton (Gtk.Stock.Cancel, ResponseType.Cancel);
			fcd.AddButton (Gtk.Stock.Save, ResponseType.Ok);

			fcd.SelectMultiple = false;

			ResponseType response = (ResponseType) fcd.Run();
			if (response == ResponseType.Ok)  {			
				string attributeName = this.CurrentAttributeName;
				LdapEntry le = conn.Data.GetEntry (currentDN);
				LdapAttribute la = le.getAttribute (attributeName);
				
				WriteBytesToFile (fcd.Filename, SupportClass.ToByteArray (la.ByteValue)); 
			}
			
			fcd.Destroy ();
		}
	}
}
