// 
// lat - GtkLdapSearch.cs
// Author: Loren Bandiera
// Copyright 2005 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 .
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using Gtk;
using GLib;
using Gdk;
using Novell.Directory.Ldap;
using Novell.Directory.Ldap.Rfc2251;
using Novell.Directory.Ldap.Utilclass;

namespace lat
{	
	public class GtkLdapSearch : Gtk.Bin
	{
		ListStore resultsStore;
		
		static TargetEntry[] searchSourceTable = new TargetEntry[]
		{
			new TargetEntry ("text/plain", 0, 1),
		};
	
		protected Gtk.ComboBox serverComboBox;
		protected Gtk.Entry filterEntry;
		protected Gtk.HBox serverHBox;
		protected Gtk.Button searchBaseButton;
		protected Gtk.TreeView resultsTreeView;

		
		public GtkLdapSearch()
		{
			Stetic.Gui.Build(this, typeof(lat.GtkLdapSearch));
			
			serverComboBox = Util.CreateServerCombo (); 
			serverHBox.PackStart (serverComboBox, true, true, 0);
					
			resultsStore = new ListStore (typeof (string));
			
			resultsTreeView.Model = resultsStore;
			resultsTreeView.HeadersVisible = false;
			resultsTreeView.AppendColumn ("resultDN", new CellRendererText (), "text", 0);			
			resultsTreeView.ButtonPressEvent += new ButtonPressEventHandler (OnRightClick);
			
			Gtk.Drag.SourceSet (resultsTreeView, Gdk.ModifierType.Button1Mask | Gdk.ModifierType.Button3Mask, 
				searchSourceTable, Gdk.DragAction.Copy | Gdk.DragAction.Move);			
		}

		public void Show (LdapEntry[] searchResults)
		{
			resultsStore.Clear ();

			if (!(searchResults.Length > 0)) {
				resultsStore.AppendValues (Mono.Unix.Catalog.GetString ("No matches found."));
				return;
			}
			
			foreach (LdapEntry le in searchResults)
				resultsStore.AppendValues (le.DN);
		}

		public string SelectedResult
		{
			get { return GetSelectedSearchResult(); }
		}

		public string SelectedServer
		{
			get { 
				TreeIter iter;
				if (!serverComboBox.GetActiveIter (out iter))
					return null;
					
				return (string) serverComboBox.Model.GetValue (iter, 0);
			}
		}

		public event SearchResultExportHandler Export;
		public event SearchResultSelectedHandler SearchResultSelected;
		public event StatusMessageSentHandler StatusMessageSent;		

		#region internal_methods
		
		void DispatchSearchResultSelectedEvent (string dn)
		{
			if (SearchResultSelected != null)
				SearchResultSelected (this, new SearchResultSelectedEventArgs (dn));
		}
		
		void DispatchStatusMessageSentEvent (string statusMessage)
		{
			if (StatusMessageSent != null)
				StatusMessageSent (this, new StatusMessageSentEventArgs (statusMessage));
		}
		
		void DoPopUp()
		{
			Menu popup = new Menu();
			MenuItem exportItem = new MenuItem ("Export...");
			exportItem.Activated += new EventHandler (OnExportActivate);
			exportItem.Show ();

			popup.Append (exportItem);

			popup.Popup(null, null, null, 3, Gtk.Global.CurrentEventTime);
		}
		
		string GetSelectedSearchResult ()
		{
			TreeModel model;
			TreeIter iter;

			if (!resultsTreeView.Selection.GetSelected (out model, out iter))
				return null;
			
			string name = (string) resultsStore.GetValue (iter, 0);
			return name;
		}
		
		#endregion

		protected virtual void OnDragBegin(object o, Gtk.DragBeginArgs args)
		{
			Gdk.Pixbuf pb = Gdk.Pixbuf.LoadFromResource ("text-x-generic.png");
			Gtk.Drag.SetIconPixbuf (args.Context, pb, 0, 0);		
		}

		protected virtual void OnDragDataGet(object o, Gtk.DragDataGetArgs args)
		{
			Gtk.TreeModel model;
			Gtk.TreeIter iter;

			if (!resultsTreeView.Selection.GetSelected (out model, out iter))
				return;

			string dn = (string) model.GetValue (iter, 0);
			
			SearchResultExportEventArgs myargs = new SearchResultExportEventArgs (dn, true);
		
			if (Export != null)
				Export (this, myargs);

			if (myargs.Data == null)
				return;
				
			Atom[] targets = args.Context.Targets;

			args.SelectionData.Set (targets[0], 8, System.Text.Encoding.UTF8.GetBytes (myargs.Data));		
		}

		void OnExportActivate (object o, EventArgs args)
		{
			Gtk.TreeModel model;
			Gtk.TreeIter iter;

			if (!resultsTreeView.Selection.GetSelected (out model, out iter))
				return;

			string dn = (string) model.GetValue (iter, 0);
			
			SearchResultExportEventArgs myargs = new SearchResultExportEventArgs (dn, false);
		
			if (Export != null)
				Export (this, myargs);		
		}

		[ConnectBefore]
		void OnRightClick (object o, ButtonPressEventArgs args)
		{
			if (args.Event.Button == 3)
				DoPopUp ();
		}

		protected virtual void OnRowActivated(object o, Gtk.RowActivatedArgs args)
		{
			DispatchSearchResultSelectedEvent (GetSelectedSearchResult ());
		}

		protected virtual void OnSearchBaseClicked(object sender, System.EventArgs e)
		{
			TreeIter iter;				
			if (!serverComboBox.GetActiveIter (out iter))
				return;

			string profileName = (string) serverComboBox.Model.GetValue (iter, 0);			
			Connection conn = Global.Connections [profileName];
		
			try {
			
				SelectContainerDialog scd = new SelectContainerDialog (conn);
				scd.Message = String.Format (Mono.Unix.Catalog.GetString ("Where in the directory would\nyou like to start the search?"));
				scd.Title = Mono.Unix.Catalog.GetString ("Select search base");
				
				ResponseType res = (ResponseType) scd.Run ();				
				if (res == ResponseType.Ok) {
					if (!scd.DN.Equals ("") && !scd.DN.Equals (conn.Settings.Host))
						searchBaseButton.Label = scd.DN;
				}
				
				scd.Destroy ();
				
			} catch (Exception ex) {
				Log.Debug (ex);
			}
		}

		protected virtual void OnSearchClicked(object sender, System.EventArgs e)
		{
			if (searchBaseButton.Label == null || searchBaseButton.Label == string.Empty) {
				Util.ShowErrorDialog ("Search error", "No search base set");
				return;
			}
		
			TreeIter iter;
			if (!serverComboBox.GetActiveIter (out iter))
				return;

			string profileName = (string) serverComboBox.Model.GetValue (iter, 0);			
			Connection conn = Global.Connections [profileName];
			
			RfcFilter filter = new RfcFilter (filterEntry.Text);			
			LdapEntry[] searchResults = conn.Data.Search (searchBaseButton.Label, filter.filterToString());

			if (searchResults == null) {
			
				Util.ShowErrorDialog ("Search error", "Invalid search filter");
				
			} else if (searchResults.Length > 0 && filterEntry.Text != "") {
			
				this.Show (searchResults);
			 
				string msg = String.Format ("Found {0} matching entries", searchResults.Length);
				DispatchStatusMessageSentEvent (msg);
			}		
		}

		protected virtual void OnSearchBuilderClicked(object sender, System.EventArgs e)
		{
			SearchBuilderDialog dlg = new SearchBuilderDialog ();
			ResponseType res = (ResponseType) dlg.Run ();
			
			if (res == ResponseType.Ok)
				filterEntry.Text = dlg.UserFilter;
			
			dlg.Destroy ();
		}
		
		
	}
	
	public class SearchResultSelectedEventArgs : EventArgs
	{
		public string DN;

		public SearchResultSelectedEventArgs (string dn)
		{
			this.DN = dn;
		}
	}

	public class SearchResultExportEventArgs : EventArgs
	{
		public string DN;
		public bool IsDND;
		public string Data;

		public SearchResultExportEventArgs (string dn, bool isdnd)
		{
			this.DN = dn;
			this.IsDND = isdnd;
		}
	}

	public delegate void SearchResultSelectedHandler (object o, SearchResultSelectedEventArgs args);
	public delegate void SearchResultExportHandler (object o, SearchResultExportEventArgs args);	
}
